var stream = require('stream');
var inherits = require('util').inherits;

function MockSTDIN(restoreTarget) {
  stream.Readable.call(this, {
    highWaterMark: 0,
    readable: true,
    writable: false
  });
  Object.defineProperties(this, {
    target: {
      enumerable: false,
      writable: false,
      configurable: false,
      value: restoreTarget
    },
    isMock: {
      enumerable: false,
      writable: false,
      configurable: false,
      value: true
    },
    _mockData: {
      enumerable: false,
      writable: false,
      configurable: false,
      value: []
    }
  });
}

inherits(MockSTDIN, stream.Readable);

function MockData(chunk) {
  Object.defineProperties(this, {
    data: {
      value: chunk,
      writable: false,
      configurable: false,
      enumerable: false
    },
    length: {
      get: function() {
        if (Buffer.isBuffer(chunk)) {
          return chunk.length;
        } else if (typeof chunk === 'string') {
          return chunk.length;
        }
        return 0;
      },
      configurable: false,
      enumerable: false
    },
    pos: {
      writable: true,
      value: 0,
      configurable: false,
      enumerable: false
    },
    done: {
      writable: true,
      value: false,
      configurable: false,
      enumerable: false
    }
  });
}

MockData.prototype.chunk = function(length) {
  if (this.pos <= this.length) {
    if (Buffer.isBuffer(this.data) || typeof this.data === 'string') {
      var value = this.data.slice(this.pos, this.pos + length);
      this.pos += length;
      if (this.pos >= this.length) {
        this.done = true;
      }
      return value;
    }
  }

  this.done = true;
  return null;
}

MockSTDIN.prototype.send = function MockSTDINWrite(text) {
  if (Array.isArray(text)) text = text.join('\n');
  if (Buffer.isBuffer(text) || typeof text === 'string' || text === null) {
    var data = new MockData(text);
    this._mockData.push(data);
    this._read();
    if (text === null) {
      // Trigger an end event synchronously...
      endReadable(this);
    }
  }
  return this;
};

MockSTDIN.prototype.end = function MockSTDINEnd() {
  this.send(null);
  return this;
};

MockSTDIN.prototype.restore = function MockSTDINRestore() {
  Object.defineProperty(process, 'stdin', {
    value: this.target,
    configurable: true,
    writable: false
  });
  return this;
};

MockSTDIN.prototype.reset = function MockSTDINReset() {
  var state = this._readableState;
  state.ended = false;
  return this;
};

MockSTDIN.prototype._read = function MockSTDINRead(size) {
  if (size === void 0) size = Infinity;
  var count = 0;
  var read = true;
  while (read && this._mockData.length && count < size) {
    var item = this._mockData[0];
    var leftInChunk = item.length - item.pos;
    var remaining = size === Infinity ? leftInChunk : size - count;
    var toProcess = Math.min(leftInChunk, remaining);

    if (!this.push(item.chunk(toProcess))) {
      read = false;
    }

    if (item.done) {
      this._mockData.shift();
    }

    count += toProcess;
  }
};

function endReadable(stream) {
  // Synchronously emit an end event, if possible.
  var state = stream._readableState;

  if (!state.length) {
    state.ended = true;
    state.endEmitted = true;
    stream.readable = false;
    stream.emit('end');
  }
}

function mock() {
  var mock = new MockSTDIN(process.stdin);
  Object.defineProperty(process, 'stdin', {
    value: mock,
    configurable: true,
    writable: false
  });
  return mock;
}

mock.Class = MockSTDIN;

module.exports = mock;
